using System;
using System.Collections.Generic;
using System.Linq;
using GTA;
using GTA.Native;
using GTA.Math;

public class ZombieInfection : Script
{
    private List<Ped> zombies = new List<Ped>();
    private List<Ped> infectedAnimals = new List<Ped>();
    private List<Ped> survivors = new List<Ped>();
    private List<Ped> survivorPets = new List<Ped>(); // Liste der tierischen Begleiter
    private List<Blip> targetBlips = new List<Blip>();
    private List<Ped> normalPeds = new List<Ped>();
    private Random random = new Random();
    private Vector3 indoorAreaCenter = new Vector3(0, 0, 0); // Beispielkoordinaten für Innenbereich
    private float indoorAreaRadius = 50f; // Radius des Innenbereichs
    private bool hasInfectedPedsOrAnimals = false;
    private bool blackoutActive = false; // Status des Blackouts
    private int tickCounter = 0;
    private const int maxPedsPerTick = 10; // Maximale Anzahl der zu verarbeitenden Peds pro Tick
    private int infectedCount = 0; // Zählt die Anzahl der infizierten Peds
    private int groupSize = 3; // Größe der Überlebendengruppen
    private Dictionary<Ped, SurvivorBehavior> survivorBehaviors = new Dictionary<Ped, SurvivorBehavior>();
    private Dictionary<Ped, DateTime> recentlyExitedVehicles = new Dictionary<Ped, DateTime>(); // Map von Peds und dem Zeitpunkt, an dem sie das Fahrzeug verlassen haben
    private Vector3 lastGunshotPosition = Vector3.Zero; // Position des letzten Schusses

    public ZombieInfection()
    {
        Tick += OnTick;
        KeyDown += OnKeyDown;
        Interval = 1000; // Tick-Intervall in Millisekunden
        ApplyBloodEffect(Game.Player.Character);
    }

    private void OnKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
    {
        if (e.KeyCode == System.Windows.Forms.Keys.O)
        {
            ToggleBlackout();
        }
        else if (e.KeyCode == System.Windows.Forms.Keys.X)
        {
            // Infizieren des nächsten Peds sofort und unabhängig von der Aktivität des Spielers
            InfectNearestPedImmediate();
        }
    }

    private void OnTick(object sender, EventArgs e)
    {
        tickCounter++;

        // Verteilt die Last auf mehrere Ticks
        if (tickCounter % 5 == 0) UpdateBlips();
        if (tickCounter % 2 == 0) UpdateZombies();
        if (tickCounter % 3 == 0) UpdateInfectedAnimals();
        if (tickCounter % 5 == 0) UpdateGangsCopsMilitaryAndAltruists();
        if (tickCounter % 6 == 0) UpdateSurvivors();
        if (tickCounter % 4 == 0) UpdateNormalPeds(); // Hinzugefügt: Update normales Ped-Verhalten

        // Überprüfe, ob der Spielercharakter oder die Kleidung gewechselt wurde
        Ped player = Game.Player.Character;
        ApplyBloodEffect(player); // Ständig anwenden, um das Blut dauerhaft zu zeigen

        // Erstelle gelegentlich Überlebende, aber nur wenn bereits Peds oder Tiere infiziert wurden
        if (hasInfectedPedsOrAnimals && random.NextDouble() < 0.10) // Erhöhte Wahrscheinlichkeit (10%) bei jedem Tick
        {
            CreateRandomSurvivor();
        }

        // Überprüfe, ob genügend Peds infiziert wurden, um Überlebende zu erzeugen
        CheckAndCreateSurvivors();

        // Überprüfe auf Schüsse in der Nähe
        CheckForGunshots();
    }

    private void ToggleBlackout()
    {
        blackoutActive = !blackoutActive;
        Function.Call(Hash._SET_BLACKOUT, blackoutActive);
        UI.Notify(blackoutActive ? "Blackout aktiviert" : "Blackout deaktiviert");
    }

    private void UpdateBlips()
    {
        int processedCount = 0;
        foreach (Ped ped in World.GetAllPeds())
        {
            if (processedCount >= maxPedsPerTick) break; // Verarbeite maximal 10 Peds pro Tick
            if (IsValidTarget(ped) && !HasBlip(ped))
            {
                AddBlip(ped);
                processedCount++;
            }
        }
    }

    private void UpdateZombies()
    {
        int processedCount = 0;
        for (int i = zombies.Count - 1; i >= 0; i--)
        {
            if (processedCount >= maxPedsPerTick) break;
            Ped zombie = zombies[i];
            if (zombie.IsDead)
            {
                zombies.RemoveAt(i);
                continue;
            }

            // Locken von Zombies zu der letzten Schussposition
            if (lastGunshotPosition != Vector3.Zero)
            {
                FollowTarget(zombie, lastGunshotPosition);
            }

            Ped target = FindClosestNonInfectedPed(zombie.Position, zombie);
            if (target != null && target != zombie)
            {
                if (CanLeaveIndoorArea(zombie))
                {
                    FollowTarget(zombie, target.Position);
                }
                else if (!IsInIndoorArea(zombie))
                {
                    FollowTarget(zombie, target.Position);
                }

                // Normale Geh-Animation für Zombies
                Function.Call(Hash.RESET_PED_MOVEMENT_CLIPSET, zombie, 0);
                Function.Call(Hash.SET_PED_CAN_RAGDOLL, zombie, true); // Ragdoll aktivieren
                Function.Call(Hash.SET_PED_FLEE_ATTRIBUTES, zombie, 0, false); // Flucht deaktivieren
                Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, zombie, 46, true); // Kämpfen immer aktivieren
                Function.Call(Hash.SET_PED_CONFIG_FLAG, zombie, 32, false); // Verhindere das Einsteigen in Fahrzeuge

                // Füge die Möglichkeit hinzu, Fahrzeuge anzugreifen, aber nicht das Fahrzeug, das sie gerade verlassen haben
                if (!zombie.IsInVehicle() && (!recentlyExitedVehicles.ContainsKey(zombie) || (DateTime.Now - recentlyExitedVehicles[zombie]).TotalSeconds > 10))
                {
                    Vehicle nearbyVehicle = World.GetClosestVehicle(zombie.Position, 5.0f);
                    if (nearbyVehicle != null && nearbyVehicle.Driver != zombie)
                    {
                        AttackVehicle(zombie, nearbyVehicle);
                    }
                }

                if (zombie.Position.DistanceTo(target.Position) < 1.5f)
                {
                    if (random.NextDouble() < 0.3) // 30% Chance, das Ziel zu infizieren
                    {
                        InfectPed(zombie, target);
                    }
                }
                else if (zombie.Position.DistanceTo(target.Position) < 10.0f)
                {
                    // Geschwindigkeit erhöhen, um das Ziel zu erreichen
                    Function.Call(Hash.SET_PED_MOVE_RATE_OVERRIDE, zombie, 2.0f);
                }
            }
            else
            {
                // Finde ein neues Ziel, wenn das aktuelle Ziel ungültig ist
                FindNewTargetForZombie(zombie);
            }
            processedCount++;
        }
    }

    private void AttackVehicle(Ped attacker, Vehicle vehicle)
    {
        if (vehicle != null && attacker != null && !attacker.IsDead && !vehicle.IsDead)
        {
            Function.Call(Hash.SET_VEHICLE_DAMAGE, vehicle, 0.5f, 0.5f, 0.5f, 1000f, 1000f, true); // Schaden am Fahrzeug
        }
    }

    private void UpdateInfectedAnimals()
    {
        int processedCount = 0;
        for (int i = infectedAnimals.Count - 1; i >= 0; i--)
        {
            if (processedCount >= maxPedsPerTick) break;
            Ped animal = infectedAnimals[i];
            if (animal.IsDead)
            {
                infectedAnimals.RemoveAt(i);
                continue;
            }

            // Locken von infizierten Tieren zu der letzten Schussposition
            if (lastGunshotPosition != Vector3.Zero)
            {
                FollowAndAttackTarget(animal, lastGunshotPosition);
            }

            Ped target = FindClosestNonInfectedPed(animal.Position, animal);
            if (target != null && target != animal)
            {
                if (CanLeaveIndoorArea(animal))
                {
                    FollowAndAttackTarget(animal, target.Position);
                }
                else if (!IsInIndoorArea(animal))
                {
                    FollowAndAttackTarget(animal, target.Position);
                }

                if (animal.Position.DistanceTo(target.Position) < 1.5f)
                {
                    PlayAttackAnimation(animal);
                    if (random.NextDouble() < 0.3) // 30% Chance, das Ziel zu infizieren
                    {
                        InfectPed(animal, target);
                    }
                }
                else if (animal.Position.DistanceTo(target.Position) < 10.0f)
                {
                    // Geschwindigkeit erhöhen, um das Ziel zu erreichen
                    Function.Call(Hash.SET_PED_MOVE_RATE_OVERRIDE, animal, 2.0f);
                }
            }
            else
            {
                // Finde ein neues Ziel, wenn das aktuelle Ziel ungültig ist
                FindNewTargetForAnimal(animal);
            }
            processedCount++;
        }
    }

    private void UpdateGangsCopsMilitaryAndAltruists()
    {
        int processedCount = 0;
        foreach (Ped ped in World.GetAllPeds())
        {
            if (processedCount >= maxPedsPerTick) break;
            if (ped.IsDead || ped.IsPlayer || zombies.Contains(ped) || infectedAnimals.Contains(ped) || survivors.Contains(ped))
                continue;

            if (IsGangMember(ped) || IsPoliceOfficer(ped) || IsMilitarySoldier(ped) || IsAltruist(ped))
            {
                Ped target = FindClosestPed(ped.Position, ped);
                if (target != null && (zombies.Contains(target) || infectedAnimals.Contains(target)))
                {
                    ped.Task.FightAgainst(target);
                    processedCount++;
                }
            }
        }
    }

    private void UpdateSurvivors()
    {
        if (!hasInfectedPedsOrAnimals) return; // Keine Überlebenden erzeugen, wenn noch nichts infiziert wurde

        int processedCount = 0;
        for (int i = survivors.Count - 1; i >= 0; i--)
        {
            if (processedCount >= maxPedsPerTick) break;
            Ped survivor = survivors[i];
            if (survivor.IsDead)
            {
                survivors.RemoveAt(i);
                continue;
            }

            if (!survivorBehaviors.ContainsKey(survivor))
            {
                // Weise dem Überlebenden ein zufälliges Verhalten zu
                survivorBehaviors[survivor] = (SurvivorBehavior)random.Next(0, Enum.GetValues(typeof(SurvivorBehavior)).Length);
            }

            switch (survivorBehaviors[survivor])
            {
                case SurvivorBehavior.Group:
                    HandleGroupBehavior(survivor, i);
                    break;
                case SurvivorBehavior.Loner:
                    HandleLonerBehavior(survivor);
                    break;
                case SurvivorBehavior.Hostile:
                    HandleHostileBehavior(survivor);
                    break;
            }

            processedCount++;
        }
    }

    private void HandleGroupBehavior(Ped survivor, int index)
    {
        if (index % groupSize == 0)
        {
            // Gruppenanführer findet Ziel
            Ped target = FindClosestEnemy(survivor.Position);
            if (target != null && !target.IsDead)
            {
                survivor.Task.ClearAll();
                survivor.Task.FightAgainst(target);
                survivor.Task.ShootAt(target, -1, FiringPattern.FullAuto);
            }
            else
            {
                // Suche nach einem neuen Ziel, wenn das aktuelle Ziel ungültig ist
                Ped newTarget = FindClosestEnemy(survivor.Position);
                if (newTarget != null && !newTarget.IsDead)
                {
                    survivor.Task.ClearAll();
                    survivor.Task.FightAgainst(newTarget);
                    survivor.Task.ShootAt(newTarget, -1, FiringPattern.FullAuto);
                }
            }

            // Überprüfe auf nahe Ziele und greife sie an
            List<Ped> nearbyEnemies = World.GetAllPeds().Where(p => (zombies.Contains(p) || infectedAnimals.Contains(p)) && p.Position.DistanceTo(survivor.Position) < 10.0f).ToList();
            foreach (Ped nearbyEnemy in nearbyEnemies)
            {
                if (!nearbyEnemy.IsDead) // Überprüfen, ob das Ziel nicht tot ist
                {
                    survivor.Task.ClearAll();
                    survivor.Task.FightAgainst(nearbyEnemy);
                    survivor.Task.ShootAt(nearbyEnemy, -1, FiringPattern.FullAuto);
                }
            }

            // Füge zufällig einen Hund hinzu
            if (random.NextDouble() < 0.3) // 30% Chance, dass die Gruppe einen Hund hat
            {
                Vector3 spawnPosition = GetSafeSpawnPosition(survivor.Position);
                if (spawnPosition != Vector3.Zero)
                {
                    PedHash[] dogHashes = { PedHash.Rottweiler, PedHash.Husky };
                    Ped dog = World.CreatePed(dogHashes[random.Next(dogHashes.Length)], spawnPosition);
                    survivorPets.Add(dog);
                    Function.Call(Hash.TASK_FOLLOW_TO_OFFSET_OF_ENTITY, dog, survivor, 1.0f, 1.0f, 0.0f, 1.0f, -1, 1.0f, true);
                }
            }

            // Füge einen Kojoten hinzu
            if (random.NextDouble() < 0.3) // 30% Chance, dass die Gruppe einen Kojoten hat
            {
                Vector3 spawnPosition = GetSafeSpawnPosition(survivor.Position);
                if (spawnPosition != Vector3.Zero)
                {
                    Ped coyote = World.CreatePed(PedHash.Coyote, spawnPosition);
                    survivorPets.Add(coyote);
                    Function.Call(Hash.TASK_FOLLOW_TO_OFFSET_OF_ENTITY, coyote, survivor, 1.0f, 1.0f, 0.0f, 1.0f, -1, 1.0f, true);
                }
            }
        }
        else
        {
            // Gruppenmitglieder folgen dem Anführer und greifen an
            Ped leader = survivors[index - (index % groupSize)];
            if (leader != null && !leader.IsDead)
            {
                survivor.Task.ClearAll();
                Function.Call(Hash.TASK_FOLLOW_TO_OFFSET_OF_ENTITY, survivor, leader, 1.5f * (index % groupSize), 0, 0, 1.0f, -1, 2.0f, true);
            }
        }
    }

    private Vector3 GetSafeSpawnPosition(Vector3 basePosition)
    {
        int[] submarineModelHashes = { 771711535, -322360362, -2270768 }; // Hashes für Kraken, Submersible, Submersible2

        for (int i = 0; i < 10; i++) // Versuche bis zu 10 Mal eine sichere Position zu finden
        {
            Vector3 spawnPosition = basePosition.Around(3.0f);
            if (!World.GetNearbyVehicles(spawnPosition, 2.0f).Any(v => v.IsOnAllWheels || v.IsInAir || v.Model.IsHelicopter || v.Model.IsPlane || v.Model.IsBoat || submarineModelHashes.Contains(v.Model.Hash)))
            {
                return spawnPosition;
            }
        }
        return Vector3.Zero; // Wenn keine sichere Position gefunden wird, gebe Vector3.Zero zurück
    }

    private void HandleLonerBehavior(Ped survivor)
    {
        Ped target = FindClosestEnemy(survivor.Position);
        if (target != null && !target.IsDead)
        {
            survivor.Task.ClearAll();
            survivor.Task.FightAgainst(target);
            survivor.Task.ShootAt(target, -1, FiringPattern.FullAuto);
        }
        else
        {
            // Suche nach einem neuen Ziel, wenn das aktuelle Ziel ungültig ist
            Ped newTarget = FindClosestEnemy(survivor.Position);
            if (newTarget != null && !newTarget.IsDead)
            {
                survivor.Task.ClearAll();
                survivor.Task.FightAgainst(newTarget);
                survivor.Task.ShootAt(newTarget, -1, FiringPattern.FullAuto);
            }
        }
    }

    private void HandleHostileBehavior(Ped survivor)
    {
        List<Ped> nearbyPeds = World.GetAllPeds().Where(p => p != survivor && p.Position.DistanceTo(survivor.Position) < 15.0f).ToList();
        foreach (Ped nearbyPed in nearbyPeds)
        {
            if (!nearbyPed.IsDead) // Überprüfen, ob das Ziel nicht tot ist
            {
                survivor.Task.ClearAll();
                survivor.Task.FightAgainst(nearbyPed);
                survivor.Task.ShootAt(nearbyPed, -1, FiringPattern.FullAuto);
            }
        }
    }

    private void UpdateNormalPeds()
    {
        foreach (Ped ped in World.GetAllPeds())
        {
            if (ped.IsPlayer || ped.IsDead || zombies.Contains(ped) || infectedAnimals.Contains(ped) || survivors.Contains(ped) || normalPeds.Contains(ped))
                continue;

            if (random.NextDouble() < 0.2) // 20% Chance, dass das Ped sich besonders verhält
            {
                normalPeds.Add(ped);

                // Zufälliges Verhalten
                double behaviorChance = random.NextDouble();
                if (behaviorChance < 0.5)
                {
                    // In Panik geraten und wegrennen
                    ped.Task.ReactAndFlee(Game.Player.Character);
                }
                else if (behaviorChance < 0.75)
                {
                    // In Fahrzeug einsteigen und wegfahren
                    Vehicle nearestVehicle = World.GetClosestVehicle(ped.Position, 10.0f);
                    if (nearestVehicle != null)
                    {
                        ped.Task.EnterVehicle(nearestVehicle, VehicleSeat.Driver);
                        // Aggressives Fahren und Unfall verursachen
                        ped.DrivingStyle = DrivingStyle.AvoidTrafficExtremely;
                        ped.DrivingSpeed = 50.0f;
                    }
                }
                else
                {
                    // Sich verstecken
                    ped.Task.ClearAll();
                    ped.Task.GoTo(new Vector3(ped.Position.X + random.Next(-10, 10), ped.Position.Y + random.Next(-10, 10), ped.Position.Z));
                }
            }
        }
    }

    private bool IsValidTarget(Ped ped)
    {
        return !ped.IsPlayer && !ped.IsDead && !zombies.Contains(ped) && !infectedAnimals.Contains(ped);
    }

    private void InfectNearestPedImmediate()
    {
        Ped player = Game.Player.Character;
        Ped target = FindClosestNonInfectedPed(player.Position, player);
        if (target != null)
        {
            // Infiziere das Ped sofort, unabhängig von der Aktivität des Spielers
            InfectPed(player, target);
        }
    }

    private Ped FindClosestNonInfectedPed(Vector3 position, Ped ignorePed)
    {
        Ped closestPed = null;
        float closestDistance = float.MaxValue;

        foreach (Ped ped in World.GetAllPeds())
        {
            if (ped == ignorePed || ped.IsPlayer || ped.IsDead || infectedAnimals.Contains(ped) || zombies.Contains(ped)) continue;

            float distance = position.DistanceTo(ped.Position);
            if (distance < closestDistance)
            {
                closestDistance = distance;
                closestPed = ped;
            }
        }

        return closestPed;
    }

    private Ped FindClosestEnemy(Vector3 position)
    {
        Ped closestPed = null;
        float closestDistance = float.MaxValue;

        foreach (Ped ped in World.GetAllPeds())
        {
            if (ped.IsDead || ped == Game.Player.Character) continue;

            bool isEnemy = zombies.Contains(ped) || infectedAnimals.Contains(ped);
            if (!isEnemy) continue;

            float distance = position.DistanceTo(ped.Position);
            if (distance < closestDistance)
            {
                closestDistance = distance;
                closestPed = ped;
            }
        }

        return closestPed;
    }

    private Ped FindClosestPed(Vector3 position, Ped ignorePed)
    {
        Ped closestPed = null;
        float closestDistance = float.MaxValue;

        foreach (Ped ped in World.GetAllPeds())
        {
            if (ped == ignorePed || ped == Game.Player.Character) continue;

            float distance = position.DistanceTo(ped.Position);
            if (distance < closestDistance)
            {
                closestDistance = distance;
                closestPed = ped;
            }
        }

        return closestPed;
    }

    private void InfectPed(Ped infector, Ped target)
    {
        if (target.IsDead)
        {
            return; // Infiziere keine toten Peds mehr
        }
        else if (IsAnimal(infector))
        {
            infectedAnimals.Add(target);
            PlayAttackAnimation(infector);
        }
        else
        {
            if (target.IsInVehicle())
            {
                target.Task.LeaveVehicle();
                recentlyExitedVehicles[target] = DateTime.Now; // Markiere das Ped als kürzlich ausgestiegen
            }

            InfectAsZombie(target);
        }

        // Mark that Peds or animals have been infected
        hasInfectedPedsOrAnimals = true;
        infectedCount++; // Erhöhe den Zähler für infizierte Peds
    }

    private void InfectAsZombie(Ped target)
    {
        // Entferne Waffen, wenn das Ziel infiziert wird
        target.Weapons.RemoveAll();

        zombies.Add(target);

        RemoveBlip(target);
        ApplyBloodEffect(target);

        if (IsStoryCharacter(target))
        {
            InterruptCurrentAction(target);
        }

        SetPedAttributes(target);

        // Entferne Überlebende aus der Survivor-Liste
        if (survivors.Contains(target))
        {
            survivors.Remove(target);
        }

        // Sofortige Aufgabenanpassung ohne Zurücksetzen
        PedFollowLogic(target);
    }

    private void PedFollowLogic(Ped follower)
    {
        Vector3 targetPos = follower.Position;
        Function.Call(Hash.TASK_GO_TO_COORD_ANY_MEANS, follower, targetPos.X, targetPos.Y, targetPos.Z, 4.0f, 0, 0, 786603, 0xbf800000);
    }

    private void SetPedAttributes(Ped target)
    {
        // Normale Geh-Animation für Läufer
        Function.Call(Hash.RESET_PED_MOVEMENT_CLIPSET, target, 0);

        target.Weapons.RemoveAll(); // Entferne alle Waffen

        // Set aggressive behavior without specific attack animations
        Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, target, 46, true); // Always fight
        Function.Call(Hash.SET_PED_COMBAT_MOVEMENT, target, 2); // Will advance
        Function.Call(Hash.SET_PED_COMBAT_RANGE, target, 2); // Medium range
        Function.Call(Hash.SET_PED_TARGET_LOSS_RESPONSE, target, 1); // Search for targets
        Function.Call(Hash.SET_PED_COMBAT_ABILITY, target, 2); // Professional
        Function.Call(Hash.SET_PED_FLEE_ATTRIBUTES, target, 0, false); // Disable fleeing
        Function.Call(Hash.SET_PED_CAN_RAGDOLL, target, true); // Enable ragdoll
        Function.Call(Hash.SET_PED_SUFFERS_CRITICAL_HITS, target, true); // Enable critical hits
        Function.Call(Hash.SET_BLOCKING_OF_NON_TEMPORARY_EVENTS, target, true); // Prevent temporary events
        Function.Call(Hash.SET_PED_CAN_EVASIVE_DIVE, target, false); // Disable evasive diving
        Function.Call(Hash.SET_PED_SHOOT_RATE, target, 1000); // High shoot rate to simulate aggressive behavior
        Function.Call(Hash.SET_PED_SEEING_RANGE, target, 1000f); // Increase seeing range
        Function.Call(Hash.SET_PED_HEARING_RANGE, target, 1000f); // Increase hearing range

        // Set attributes to prevent panicking
        Function.Call(Hash.SET_PED_FLEE_ATTRIBUTES, target, 0, true); // Disable fleeing
        Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, target, 5, true); // Always fight
        Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, target, 17, false); // Disable panic
        Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, target, 46, true); // Disable fleeing when shot
        Function.Call(Hash.SET_PED_CONFIG_FLAG, target, 118, false); // Disable panicking in response to explosions
        Function.Call(Hash.SET_PED_CONFIG_FLAG, target, 208, true); // Disable ragdoll on hit by vehicle
        Function.Call(Hash.SET_PED_CONFIG_FLAG, target, 188, true); // Disable reactions to being shot
        Function.Call(Hash.STOP_PED_SPEAKING, target, true); // Stop ped from speaking

        // Prevent entering vehicles
        Function.Call(Hash.SET_PED_CONFIG_FLAG, target, 32, false); // Prevent entering vehicles
    }

    private void PlayAttackAnimation(Ped animal)
    {
        Function.Call(Hash.TASK_PLAY_ANIM, animal, "creatures@rottweiler@melee@", "dog_takedown_bite", 8.0f, -8.0f, -1, 1, 0, false, false, false);
        Function.Call(Hash.APPLY_DAMAGE_TO_PED, animal, 10, true); // Damage the target to simulate an attack
    }

    private bool IsAnimal(Ped ped)
    {
        PedHash[] animalHashes = {
            PedHash.Boar, PedHash.Cat, PedHash.ChickenHawk, PedHash.Chimp, PedHash.Chop, PedHash.Cormorant, PedHash.Cow,
            PedHash.Coyote, PedHash.Crow, PedHash.Deer, PedHash.Dolphin, PedHash.Fish, PedHash.Hen, PedHash.Husky,
            PedHash.MountainLion, PedHash.Pig, PedHash.Pigeon, PedHash.Poodle, PedHash.Pug, PedHash.Rabbit, PedHash.Rat,
            PedHash.Retriever, PedHash.Rhesus, PedHash.Rottweiler, PedHash.Seagull, PedHash.Shepherd, PedHash.Stingray, PedHash.Westy
        };
        return Array.Exists(animalHashes, element => element == (PedHash)ped.Model.Hash);
    }

    private bool IsGangMember(Ped ped)
    {
        return ped.Model == PedHash.BallaEast01GMY ||
               ped.Model == PedHash.BallaOrig01GMY ||
               ped.Model == PedHash.BallaSout01GMY ||
               ped.Model == PedHash.MexGoon01GMY ||
               ped.Model == PedHash.MexGoon02GMY ||
               ped.Model == PedHash.MexGoon03GMY;
    }

    private bool IsPoliceOfficer(Ped ped)
    {
        return ped.Model == PedHash.Cop01SFY ||
               ped.Model == PedHash.Cop01SMY ||
               ped.Model == PedHash.Sheriff01SFY ||
               ped.Model == PedHash.Sheriff01SMY;
    }

    private bool IsMilitarySoldier(Ped ped)
    {
        return ped.Model == PedHash.Marine01SMY ||
               ped.Model == PedHash.Marine02SMY ||
               ped.Model == PedHash.Marine03SMY;
    }

    private bool IsAltruist(Ped ped)
    {
        return ped.Model == (PedHash)0x4E7DAE1F; // Modell-Hash für Altruist
    }

    private bool IsStoryCharacter(Ped ped)
    {
        return ped.Model == PedHash.AmandaTownley ||
               ped.Model == PedHash.Fabien ||
               ped.Model == PedHash.Floyd ||
               ped.Model == PedHash.JimmyDisanto ||
               ped.Model == PedHash.LamarDavis ||
               ped.Model == PedHash.TracyDisanto;
    }

    private void InterruptCurrentAction(Ped ped)
    {
        ped.Task.ClearAllImmediately();
    }

    private void ApplyBloodEffect(Ped ped)
    {
        Function.Call(Hash.APPLY_PED_DAMAGE_PACK, ped, "BigHitByVehicle", 0.0f, 1.0f);
        Function.Call(Hash.APPLY_PED_DAMAGE_PACK, ped, "SCR_Torture", 0.0f, 1.0f);
        Function.Call(Hash.APPLY_PED_DAMAGE_PACK, ped, "Explosion_Med", 0.0f, 1.0f);
    }

    private void AddBlip(Ped ped)
    {
        Blip blip = ped.AddBlip();
        blip.Color = BlipColor.Red;
        blip.Scale = 0.8f;
        blip.IsShortRange = true;
        targetBlips.Add(blip);
    }

    private void RemoveBlip(Ped ped)
    {
        Blip blip = ped.CurrentBlip;
        if (blip != null)
        {
            blip.Remove();
            targetBlips.Remove(blip);
        }
    }

    private bool HasBlip(Ped ped)
    {
        return ped.CurrentBlip != null;
    }

    private void FollowTarget(Ped follower, Vector3 targetPos)
    {
        Function.Call(Hash.TASK_GO_TO_COORD_ANY_MEANS, follower, targetPos.X, targetPos.Y, targetPos.Z, 4.0f, 0, 0, 786603, 0xbf800000);
    }

    private void FollowAndAttackTarget(Ped animal, Vector3 targetPos)
    {
        animal.Task.ClearAll();
        Function.Call(Hash.TASK_GO_TO_COORD_ANY_MEANS, animal, targetPos.X, targetPos.Y, targetPos.Z, 2.0f, 0, 0, 786603, 0xbf800000);
    }

    private bool IsInIndoorArea(Ped ped)
    {
        return ped.Position.DistanceTo(indoorAreaCenter) <= indoorAreaRadius;
    }

    private bool CanLeaveIndoorArea(Ped ped)
    {
        foreach (Ped otherPed in World.GetAllPeds())
        {
            if (otherPed != ped && IsInIndoorArea(otherPed) && !IsInfected(otherPed))
            {
                return false;
            }
        }
        return true;
    }

    private bool IsInfected(Ped ped)
    {
        return zombies.Contains(ped) || infectedAnimals.Contains(ped);
    }

    private void CreateRandomSurvivor()
    {
        Ped ped = GetRandomPed();
        if (ped != null && IsValidSurvivor(ped))
        {
            if (ped.IsInVehicle())
            {
                ped.Task.LeaveVehicle();
                recentlyExitedVehicles[ped] = DateTime.Now; // Markiere das Ped als kürzlich ausgestiegen
            }

            survivors.Add(ped);
            GiveRandomWeapon(ped);
            SetSurvivorAttributes(ped);
        }
    }

    private Ped GetRandomPed()
    {
        List<Ped> allPeds = World.GetAllPeds().Where(p => IsValidSurvivor(p)).ToList();
        if (allPeds.Count > 0)
        {
            return allPeds[random.Next(allPeds.Count)];
        }
        return null;
    }

    private bool IsValidSurvivor(Ped ped)
    {
        return !ped.IsPlayer && !ped.IsDead && !zombies.Contains(ped) && !infectedAnimals.Contains(ped) && !survivors.Contains(ped) && !IsAnimal(ped);
    }

    private void GiveRandomWeapon(Ped ped)
    {
        WeaponHash[] meleeWeapons = {
            WeaponHash.Bat, WeaponHash.Crowbar, WeaponHash.Hammer, WeaponHash.Knife, WeaponHash.Machete, WeaponHash.Wrench
        };

        WeaponHash[] rangedWeapons = {
            WeaponHash.Pistol, WeaponHash.CombatPistol, WeaponHash.MicroSMG, WeaponHash.PumpShotgun, WeaponHash.AssaultRifle
        };

        if (random.NextDouble() < 0.5) // 50% Chance für Nahkampfwaffen
        {
            ped.Weapons.Give(meleeWeapons[random.Next(meleeWeapons.Length)], 1, true, true);
        }
        else // 50% Chance für Schusswaffen
        {
            ped.Weapons.Give(rangedWeapons[random.Next(rangedWeapons.Length)], random.Next(30, 121), true, true);
        }
    }

    private void SetSurvivorAttributes(Ped ped)
    {
        Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, ped, 46, true); // Always fight
        Function.Call(Hash.SET_PED_COMBAT_MOVEMENT, ped, 2); // Will advance
        Function.Call(Hash.SET_PED_COMBAT_RANGE, ped, 2); // Medium range
        Function.Call(Hash.SET_PED_TARGET_LOSS_RESPONSE, ped, 1); // Search for targets
        Function.Call(Hash.SET_PED_COMBAT_ABILITY, ped, 2); // Professional
        Function.Call(Hash.SET_PED_FLEE_ATTRIBUTES, ped, 0, false); // Disable fleeing
        Function.Call(Hash.SET_PED_CAN_RAGDOLL, ped, true); // Enable ragdoll
        Function.Call(Hash.SET_PED_SUFFERS_CRITICAL_HITS, ped, true); // Enable critical hits
        Function.Call(Hash.SET_BLOCKING_OF_NON_TEMPORARY_EVENTS, ped, true); // Prevent temporary events
        Function.Call(Hash.SET_PED_CAN_EVASIVE_DIVE, ped, false); // Disable evasive diving
        Function.Call(Hash.SET_PED_SHOOT_RATE, ped, 1000); // High shoot rate to simulate aggressive behavior
        Function.Call(Hash.SET_PED_SEEING_RANGE, ped, 1000f); // Increase seeing range
        Function.Call(Hash.SET_PED_HEARING_RANGE, ped, 1000f); // Increase hearing range

        // Set attributes to prevent panicking
        Function.Call(Hash.SET_PED_FLEE_ATTRIBUTES, ped, 0, true); // Disable fleeing
        Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, ped, 5, true); // Always fight
        Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, ped, 17, false); // Disable panic
        Function.Call(Hash.SET_PED_COMBAT_ATTRIBUTES, ped, 46, true); // Disable fleeing when shot
        Function.Call(Hash.SET_PED_CONFIG_FLAG, ped, 118, false); // Disable panicking in response to explosions
        Function.Call(Hash.SET_PED_CONFIG_FLAG, ped, 208, true); // Disable ragdoll on hit by vehicle
        Function.Call(Hash.SET_PED_CONFIG_FLAG, ped, 188, true); // Disable reactions to being shot
        Function.Call(Hash.STOP_PED_SPEAKING, ped, true); // Stop ped from speaking

        // Prevent entering vehicles
        Function.Call(Hash.SET_PED_CONFIG_FLAG, ped, 32, false); // Prevent entering vehicles
    }

    private void FindNewTargetForZombie(Ped zombie)
    {
        Ped newTarget = FindClosestNonInfectedPed(zombie.Position, zombie);
        if (newTarget != null && newTarget != zombie)
        {
            FollowTarget(zombie, newTarget.Position);
        }
    }

    private void FindNewTargetForAnimal(Ped animal)
    {
        Ped newTarget = FindClosestNonInfectedPed(animal.Position, animal);
        if (newTarget != null && newTarget != animal)
        {
            FollowAndAttackTarget(animal, newTarget.Position);
        }
    }

    private void CheckAndCreateSurvivors()
    {
        // Bedingung, um Überlebende zu erzeugen, wenn mindestens ein Ped infiziert wurde
        if (hasInfectedPedsOrAnimals && infectedCount >= 1)
        {
            // Überprüfe, ob bereits genügend Überlebende existieren
            if (survivors.Count < 5) // Zum Beispiel, maximal 5 Überlebende
            {
                if (random.NextDouble() < 0.10) // Wahrscheinlichkeit (10%) für die Erstellung eines Überlebenden
                {
                    CreateRandomSurvivor();
                }
                infectedCount = 0; // Zähler zurücksetzen
            }
        }
    }

    private void CheckForGunshots()
    {
        foreach (Ped survivor in survivors)
        {
            if (survivor.Weapons.Current.Hash != WeaponHash.Unarmed && Function.Call<bool>(Hash.IS_PED_SHOOTING, survivor))
            {
                lastGunshotPosition = survivor.Position;
                break;
            }
        }

        if (Game.Player.Character.Weapons.Current.Hash != WeaponHash.Unarmed && Function.Call<bool>(Hash.IS_PED_SHOOTING, Game.Player.Character))
        {
            lastGunshotPosition = Game.Player.Character.Position;
        }
    }

    private enum SurvivorBehavior
    {
        Group,
        Loner,
        Hostile
    }
}
